"""/*  checker.c
 *  This program texture maps a checkerboard image onto
 *  two rectangles.
 *
 *  If running this program on OpenGL 1.0, texture objects are
 *  not used.
 */"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import numpy 
from array import *

#/*	Create checkerboard texture	*/
checkImageWidth = 64
checkImageHeight = 64
checkImage = numpy.array(bytearray(checkImageWidth*checkImageHeight*4))
texName = 1,

def makeCheckImage(checkImageHeight,checkImageWidth):
    global checkImage
    def ind(i,j,k): return (i*checkImageWidth +j)*4 + k
    
    for i in range(checkImageHeight): 
        for j in range(checkImageWidth) :
            # [i & 0x8 for i in range(64)]
            a,b = (i & 0x8 == 0)*1, (j & 0x8 == 0)*1
            if (a and not b) or (b and not a): c = 255
            else: c = 0
            # c = ((((i&0x8)==0)^((j&0x8))==0))*255;
            checkImage[ind(i,j,0)] =  c
            checkImage[ind(i,j,1)] =  c
            checkImage[ind(i,j,2)] =  c
            checkImage[ind(i,j,3)] =  255

def init():
   glClearColor (1.0, 0.0, 0.0, 0.0)
   glShadeModel(GL_FLAT)
   glEnable(GL_DEPTH_TEST)

   makeCheckImage(checkImageHeight,checkImageWidth)
   print checkImage[0:-1]
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1)

   glGenTextures(1, texName);
   glBindTexture(GL_TEXTURE_2D, texName[0]);

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, checkImageWidth, checkImageHeight, 
                0, GL_RGBA, GL_UNSIGNED_BYTE, checkImage)
   print "OK"

def display():
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
   glEnable(GL_TEXTURE_2D)
   glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL)
   glBindTexture(GL_TEXTURE_2D, texName[0]);
   print glIsTexture(texName[0])==GL_TRUE

   glBegin(GL_QUADS)
   glTexCoord2f(0.0, 0.0); glVertex3f(-2.0, -1.0, 0.0)
   glTexCoord2f(0.0, 1.0); glVertex3f(-2.0, 1.0, 0.0)
   glTexCoord2f(1.0, 1.0); glVertex3f(0.0, 1.0, 0.0)
   glTexCoord2f(1.0, 0.0); glVertex3f(0.0, -1.0, 0.0)

   glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, 0.0)
   glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 1.0, 0.0)
   glTexCoord2f(1.0, 1.0); glVertex3f(2.41421, 1.0, -1.41421)
   glTexCoord2f(1.0, 0.0); glVertex3f(2.41421, -1.0, -1.41421)
   glEnd()
   glFlush()
   glDisable(GL_TEXTURE_2D)

def reshape(w, h):
   glViewport(0, 0, w, h)
   glMatrixMode(GL_PROJECTION)
   glLoadIdentity()
   gluPerspective(60.0, float(w)/float(h), 1.0, 30.0)
   glMatrixMode(GL_MODELVIEW)
   glLoadIdentity()
   glTranslatef(0.0, 0.0, -3.6)


def keyboard(key, x, y):
   if key == chr(27):
      sys.exit(0)

# Main program entry point
if __name__ == "__main__":
   glutInit()
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH)
   glutInitWindowSize(250, 250)
   glutInitWindowPosition(100, 100)
   glutCreateWindow("Checker")
   init()
   glutDisplayFunc(display)
   glutReshapeFunc(reshape)
   glutKeyboardFunc(keyboard)
   glutMainLoop()
