"""/*
 *  light.c
 *  This program demonstrates the use of the OpenGL lighting
 *  model.  A sphere is drawn using a grey material characteristic.
 *  A single light source illuminates the object.
 */"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

"""/*  Initialize material property, light source, lighting model,
 *  and depth buffer.
 */"""
def init():

   mat_specular =  1.0, 1.0, 1.0, 1.0 
   mat_shininess =  50.0 
   light_position =  1.0, 1.0, 1.0, 0.0 

   glClearColor (0.0, 0.0, 0.0, 0.0)
   glShadeModel (GL_SMOOTH)

   ambient =  0.7, 0.7, 0.7, 1.0 
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient)
   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular)
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess)
   glLightfv(GL_LIGHT0, GL_POSITION, light_position)

   glEnable(GL_LIGHTING)
   glEnable(GL_LIGHT0)
   glEnable(GL_DEPTH_TEST)


def display():

   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
   glutSolidSphere (1.0, 40, 30)
   glFlush ()


def reshape (w, h):

   glViewport (0, 0, w, h)
   glMatrixMode (GL_PROJECTION)
   glLoadIdentity()
   if (w <= h):
      glOrtho (-1.5, 1.5, -1.5*float(h)/float(w),
         1.5*float(h)/float(w), -10.0, 10.0)
   else:
      glOrtho (-1.5*float(w)/float(h),
         1.5*float(w)/float(h), -1.5, 1.5, -10.0, 10.0)
   glMatrixMode(GL_MODELVIEW)
   glLoadIdentity()


# Main program entry point
if __name__ == "__main__":
   glutInit()
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH)
   glutInitWindowSize (500, 500) 
   glutInitWindowPosition (100, 100)
   glutCreateWindow ("Ligth")
   init ()
   glutDisplayFunc(display) 
   glutReshapeFunc(reshape)
   glutMainLoop()

